@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2005 Zope Corporation and Contributors.
+# Copyright (c) 2005 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -12,22 +12,16 @@
 #
 ##############################################################################
 """Mechanize-based Functional Doctest interfaces
-
-$Id: browser.py 109680 2010-03-05 15:56:34Z fdrake $
 """
 
 __docformat__ = "reStructuredText"
 
 
 import cStringIO
-import datetime
-import operator
 import re
 import sys
 import time
-import urllib2
 
-import ClientForm
 import mechanize
 import zope.interface
 
@@ -39,11 +33,15 @@ RegexType = type(re.compile(''))
 _compress_re = re.compile(r"\s+")
 compressText = lambda text: _compress_re.sub(' ', text.strip())
 
-def disambiguate(intermediate, msg, index):
+def disambiguate(intermediate, msg, index, choice_repr=None):
     if intermediate:
         if index is None:
             if len(intermediate) > 1:
-                raise ClientForm.AmbiguityError(msg)
+                if choice_repr:
+                    msg += ' matches:' + ''.join([
+                                '\n  %s' % choice_repr(choice)
+                                for choice in intermediate])
+                raise mechanize.AmbiguityError(msg)
             else:
                 return intermediate[0]
         else:
@@ -53,8 +51,17 @@ def disambiguate(intermediate, msg, index):
                 msg = '%s index %d' % (msg, index)
     raise LookupError(msg)
 
+def control_form_tuple_repr((ctrl, form)):
+    if isinstance(ctrl, mechanize._form.Control):
+        # mechanize._form controls have a useful __str__ and a useless __repr__
+        return str(ctrl)
+    else:
+        # mechanize._form list control items have a useful __repr__ and a
+        # too-terse __str__.
+        return repr(ctrl)
+
 def controlFactory(control, form, browser):
-    if isinstance(control, ClientForm.Item):
+    if isinstance(control, mechanize.Item):
         # it is a subcontrol
         return ItemControl(control, form, browser)
     else:
@@ -218,16 +225,21 @@ class Browser(SetattrErrorsMixin):
 
         def get(self):
             headers = self.mech_browser.addheaders
-            return dict(headers).get(header_key, True)
+            value = dict(headers).get(header_key, True)
+            return {'False': False}.get(value, True)
 
         def set(self, value):
             headers = self.mech_browser.addheaders
             current_value = get(self)
             if current_value == value:
                 return
-            if header_key in dict(headers):
-                headers.remove((header_key, current_value))
-            headers.append((header_key, value))
+
+            # Remove the current header...
+            for key, header_value in headers[:]:
+                if key == header_key:
+                    headers.remove((key, header_value))
+            # ... Before adding the new one.
+            headers.append((header_key, {False: 'False'}.get(value, 'True')))
 
         return property(get, set)
 
@@ -242,7 +254,7 @@ class Browser(SetattrErrorsMixin):
                 except Exception, e:
                     fix_exception_name(e)
                     raise
-            except urllib2.HTTPError, e:
+            except mechanize.HTTPError, e:
                 if e.code >= 200 and e.code <= 299:
                     # 200s aren't really errors
                     pass
@@ -257,7 +269,7 @@ class Browser(SetattrErrorsMixin):
             code, msg = self.headers['Status'].split(' ', 1)
             code = int(code)
             if self.raiseHttpErrors and code >= 400:
-                raise urllib2.HTTPError(url, code, msg, self.headers, fp=None)
+                raise mechanize.HTTPError(url, code, msg, self.headers, fp=None)
 
     def post(self, url, data, content_type=None):
         if content_type is not None:
@@ -369,7 +381,8 @@ class Browser(SetattrErrorsMixin):
         """See zope.testbrowser.interfaces.IBrowser"""
         intermediate, msg = self._get_all_controls(
             label, name, self.mech_browser.forms(), include_subcontrols=True)
-        control, form = disambiguate(intermediate, msg, index)
+        control, form = disambiguate(intermediate, msg, index,
+                                     control_form_tuple_repr)
         return controlFactory(control, form, self)
 
     def _get_all_controls(self, label, name, forms, include_subcontrols=False):
@@ -411,14 +424,17 @@ class Browser(SetattrErrorsMixin):
             label = labels[0].text
         else:
             label = None
-        self._start_timer()
         try:
-            self.mech_browser.open(form.click(
-                id=control.id, name=control.name, label=label, coord=coord))
-        except Exception, e:
-            fix_exception_name(e)
-            raise
-        self._stop_timer()
+            self._start_timer()
+            try:
+                self.mech_browser.form = form
+                self.mech_browser.submit(id=control.id, name=control.name,
+                                         label=label, coord=coord)
+            except Exception, e:
+                fix_exception_name(e)
+                raise
+        finally:
+            self._stop_timer()
 
     def _changed(self):
         self._counter += 1
@@ -438,9 +454,15 @@ class Link(SetattrErrorsMixin):
         if self._browser_counter != self.browser._counter:
             raise zope.testbrowser.interfaces.ExpiredError
         self.browser._start_timer()
-        self.browser.mech_browser.follow_link(self.mech_link)
-        self.browser._stop_timer()
-        self.browser._changed()
+        try:
+            try:
+                self.browser.mech_browser.follow_link(self.mech_link)
+            except Exception, e:
+                fix_exception_name(e)
+                raise
+        finally:
+            self.browser._stop_timer()
+            self.browser._changed()
 
     @property
     def url(self):
@@ -479,7 +501,7 @@ class Control(SetattrErrorsMixin):
             self.filename = None
             self.content_type = None
 
-        # for some reason ClientForm thinks we shouldn't be able to modify
+        # for some reason mechanize thinks we shouldn't be able to modify
         # hidden fields, but while testing it is sometimes very important
         if self.mech_control.type == 'hidden':
             self.mech_control.readonly = False
@@ -552,7 +574,7 @@ class ListControl(Control):
     def displayValue():
         """See zope.testbrowser.interfaces.IListControl"""
         # not implemented for anything other than select;
-        # would be nice if ClientForm implemented for checkbox and radio.
+        # would be nice if mechanize implemented for checkbox and radio.
         # attribute error for all others.
 
         def fget(self):
@@ -616,7 +638,8 @@ class ListControl(Control):
             options = self.mech_control.get_items(name=value)
             msg = 'value %r' % value
         res = controlFactory(
-            disambiguate(options, msg, index), self.mech_form, self.browser)
+            disambiguate(options, msg, index, control_form_tuple_repr),
+            self.mech_form, self.browser)
         res.__dict__['control'] = self
         return res
 
@@ -627,8 +650,10 @@ class SubmitControl(Control):
     def click(self):
         if self._browser_counter != self.browser._counter:
             raise zope.testbrowser.interfaces.ExpiredError
-        self.browser._clickSubmit(self.mech_form, self.mech_control, (1,1))
-        self.browser._changed()
+        try:
+            self.browser._clickSubmit(self.mech_form, self.mech_control, (1,1))
+        finally:
+            self.browser._changed()
 
 
 class ImageControl(Control):
@@ -637,8 +662,10 @@ class ImageControl(Control):
     def click(self, coord=(1,1)):
         if self._browser_counter != self.browser._counter:
             raise zope.testbrowser.interfaces.ExpiredError
-        self.browser._clickSubmit(self.mech_form, self.mech_control, coord)
-        self.browser._changed()
+        try:
+            self.browser._clickSubmit(self.mech_form, self.mech_control, coord)
+        finally:
+            self.browser._changed()
 
 
 class ItemControl(SetattrErrorsMixin):
@@ -701,7 +728,7 @@ class Form(SetattrErrorsMixin):
         """Initialize the Form
 
         browser - a Browser instance
-        form - a ClientForm instance
+        form - a mechanize.HTMLForm instance
         """
         self.browser = browser
         self.mech_form = form
@@ -734,23 +761,32 @@ class Form(SetattrErrorsMixin):
         if self._browser_counter != self.browser._counter:
             raise zope.testbrowser.interfaces.ExpiredError
         form = self.mech_form
-        if label is not None or name is not None:
-            intermediate, msg = self.browser._get_all_controls(
-                label, name, (form,))
-            intermediate = [
-                (control, form) for (control, form) in intermediate if
-                control.type in ('submit', 'submitbutton', 'image')]
-            control, form = disambiguate(intermediate, msg, index)
-            self.browser._clickSubmit(form, control, coord)
-        else: # JavaScript sort of submit
-            if index is not None or coord != (1,1):
-                raise ValueError(
-                    'May not use index or coord without a control')
-            request = self.mech_form._switch_click("request", urllib2.Request)
-            self.browser._start_timer()
-            self.browser.mech_browser.open(request)
-            self.browser._stop_timer()
-        self.browser._changed()
+        try:
+            if label is not None or name is not None:
+                intermediate, msg = self.browser._get_all_controls(
+                    label, name, (form,))
+                intermediate = [
+                    (control, form) for (control, form) in intermediate if
+                    control.type in ('submit', 'submitbutton', 'image')]
+                control, form = disambiguate(intermediate, msg, index,
+                                             control_form_tuple_repr)
+                self.browser._clickSubmit(form, control, coord)
+            else: # JavaScript sort of submit
+                if index is not None or coord != (1,1):
+                    raise ValueError(
+                        'May not use index or coord without a control')
+                request = self.mech_form._switch_click("request", mechanize.Request)
+                self.browser._start_timer()
+                try:
+                    try:
+                        self.browser.mech_browser.open(request)
+                    except Exception, e:
+                        fix_exception_name(e)
+                        raise
+                finally:
+                    self.browser._stop_timer()
+        finally:
+            self.browser._changed()
 
     def getControl(self, label=None, name=None, index=None):
         """See zope.testbrowser.interfaces.IBrowser"""
@@ -758,5 +794,6 @@ class Form(SetattrErrorsMixin):
             raise zope.testbrowser.interfaces.ExpiredError
         intermediate, msg = self.browser._get_all_controls(
             label, name, (self.mech_form,), include_subcontrols=True)
-        control, form = disambiguate(intermediate, msg, index)
+        control, form = disambiguate(intermediate, msg, index,
+                                     control_form_tuple_repr)
         return controlFactory(control, form, self.browser)
