@@ -11,13 +11,3 @@
 # FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
-
-class Echo:
-    """Simply echo the contents of the request"""
-
-    def __init__(self, context, request):
-        self.request = request
-
-    def __call__(self):
-        return ('\n'.join('%s: %s' % x for x in self.request.items()) +
-            '\nBody: %r' % self.request.bodyStream.read())
